#pragma once

#include <vector>
#include <Engine/types/InstancedTexture.h>
#include <Engine/types/entity/Entity.h>
#include <Engine/types/entity/Hud.h>
#include <JGL/types/RenderTarget.h>

namespace Engine {
    class Camera;
    class Scene;
}

class Engine::Scene {
protected:
    std::string name;
    Camera* active_camera = nullptr;
    Entity* entity_list = nullptr;

    std::vector<InstancedTexture*> instanced_textures{};
    std::vector<InstancedTexture*> instanced_alpha_masks{};
public:
    [[nodiscard]] Entity* GetEntityList() { return entity_list; }
    [[nodiscard]] size_t EntityCount() const;
    [[nodiscard]] std::string GetName() const;
    [[nodiscard]] Camera* GetActiveCamera() const;
    [[nodiscard]] InstancedTexture* GetInstancedTexture(const InstancedSprite* user);
    [[nodiscard]] InstancedTexture* GetInstancedAlphaMask(const InstancedSprite* user);
public:
    void SetActiveCamera(Camera* camera) { active_camera = camera; };
    void DestroyInstancedTexture(const InstancedTexture* itx);
    void DestroyInstancedAlphaMask(const InstancedTexture* alpha_mask);

    virtual void Unload();
    virtual void Init() {}
    virtual void Update();
    virtual void Render(JGL::RenderTarget* render_target = nullptr);
public:
    explicit Scene(const std::string& name) : name(name) { entity_list = new Entity(); }
    virtual ~Scene();
};
